from influxdb_client import InfluxDBClient
import matplotlib.pyplot as plt
import pandas as pd
import json

# Load configuration from config.json
config_path = r"C:\Allianz\4\0211\1201\config.json"
with open(config_path, 'r') as config_file:
    config = json.load(config_file)

url = config["url"]
token = config["token"]
org = config["org"]
bucket = config["bucket"]

try:
    # Create an InfluxDB client
    client = InfluxDBClient(url=url, token=token, org=org)

    # Query data from InfluxDB
    query = f'from(bucket:"{bucket}") |> range(start: 2023-08-01T00:00:00Z, stop: 2023-08-02T00:00:00Z)'
    tables = client.query_api().query(query, org=org)

    # Extract points from the result
    extracted_data = []
    for table in tables:
        for record in table.records:
            url_hostname = record.values.get("url_hostname")
            timestamp = record.get_time()  # Timestamp of the record
            if url_hostname and timestamp:
                extracted_data.append({"url_hostname": url_hostname, "timestamp": pd.to_datetime(timestamp)})

    # Convert to DataFrame
    df = pd.DataFrame(extracted_data)

    # Extract hour from the timestamp
    df["hour"] = df["timestamp"].dt.hour

    # Group by URL and hour to count visits
    df_grouped = df.groupby(["url_hostname", "hour"]).size().reset_index(name="visit_count")

    # Ensure all hours are present (0-24), filling missing hours with 0 visits
    all_hours = pd.DataFrame({"hour": range(25)})
    df_full = pd.DataFrame()
    for url in df_grouped["url_hostname"].unique():
        df_url = df_grouped[df_grouped["url_hostname"] == url]
        df_url_full = pd.merge(all_hours, df_url, on="hour", how="left")
        df_url_full["url_hostname"] = url
        df_url_full["visit_count"] = df_url_full["visit_count"].fillna(0).astype(int)
        df_full = pd.concat([df_full, df_url_full])

    # Define a function for plotting
    def plot_chart(df_subset, title):
        plt.figure(figsize=(14, 7))
        for url in df_subset["url_hostname"].unique():
            df_url = df_subset[df_subset["url_hostname"] == url]
            plt.plot(
                df_url["hour"], 
                df_url["visit_count"], 
                color="blue",  # Same color for all lines
                linewidth=0.8  # Thin lines
            )

        # Customize x-axis for 24 hours
        plt.xticks(range(df_subset["hour"].min(), df_subset["hour"].max() + 1), 
                   [f"{hour}:00" for hour in range(df_subset["hour"].min(), df_subset["hour"].max() + 1)], 
                   rotation=45)
        plt.xlabel("Hour of the Day", fontsize=12)
        plt.ylabel("Number of Visits", fontsize=12)
        plt.title(title, fontsize=14)
        plt.tight_layout()
        plt.show()

    # 1. Plot for night data (00:00 to 04:00)
    df_night = df_full[(df_full["hour"] >= 0) & (df_full["hour"] <= 4)]
    plot_chart(df_night, "Number of Visit URLs by Hour (Night: 00:00 - 04:00)")

    # 2. Plot for day data (04:00 to 24:00)
    df_day = df_full[(df_full["hour"] >= 4) & (df_full["hour"] <= 24)]
    plot_chart(df_day, "Number of Visit URLs by Hour (Day: 04:00 - 24:00)")
    
    # 3. Plot for all data (00:00 to 24:00)
    plot_chart(df_full, "Number of Visit URLs by Hour (All Day: 00:00 - 24:00)")


except Exception as e:
    print(f"An error occurred: {e}")
